/* Timeframe
 * Copyright (c) 2014 ROSA  <support@rosalab.ru>
 * Authors:
 *  Sergey Borovkov <sergey.borovkov@osinit.ru>
 *  Evgeniy Auzhin <evgeniy.augin@osinit.ru>
 *  Julia Mineeva <julia.mineeva@osinit.ru>
 *  Andrey Bondrov <andrey.bondrov@rosalab.ru>
 * License: GPLv3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 3,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "baloosource.h"
#include "activity.h"

#include <KMimeType>
#include <KToolInvocation>
#include <baloo/indexerconfig.h>
#include <baloo/query.h>
#include <baloo/term.h>

#include <QtCore/QDate>
#include <QtCore/QFileInfo>


BalooSource::BalooSource(QObject *parent) :
    QObject(parent)
{
    qRegisterMetaType< QList<Activity *> >("QList<Activity*>");
}

void BalooSource::startSearch()
{
    QList<Activity *> activities;
    int i = 0;
    Baloo::Query q;
    q.setTerm(Baloo::Term("modified", QDate::currentDate(), Baloo::Term::LessEqual));

// We currently cannot query multiple types, so query them one by one
    q.setType("Image");
    Baloo::ResultIterator it = q.exec();
    while (it.next()) {
        QString path = it.url().toLocalFile();
        QDate lastModified = QFileInfo(path).lastModified().date();
        KMimeType::Ptr typePtr = KMimeType::findByUrl(it.url());
        QString mimeType = typePtr->name();

        i++;

        QString type = "Image";
// Hardcode for now, 100x100 is enough
        int imageWidth = 100, imageHeight = 100;
        if (i %= 100) {
            emit newActivities(activities);
            activities.clear();
        }

        Activity *activity = new Activity(path, type, lastModified, mimeType, QSize(imageWidth, imageHeight));
        activities.append(activity);
    }

    q.setType("Video");
    it = q.exec();
    while (it.next()) {
        QString path = it.url().toLocalFile();
        QDate lastModified = QFileInfo(path).lastModified().date();
        KMimeType::Ptr typePtr = KMimeType::findByUrl(it.url());
        QString mimeType = typePtr->name();

        i++;

        QString type = "Video";
        int imageWidth = 0, imageHeight = 0;
        if (i %= 100) {
            emit newActivities(activities);
            activities.clear();
        }

        Activity *activity = new Activity(path, type, lastModified, mimeType, QSize(imageWidth, imageHeight));
        activities.append(activity);
    }

    q.setType("Document");
    it = q.exec();
    while (it.next()) {
        QString path = it.url().toLocalFile();
        QDate lastModified = QFileInfo(path).lastModified().date();
        KMimeType::Ptr typePtr = KMimeType::findByUrl(it.url());
        QString mimeType = typePtr->name();

        i++;

        QString type = "Document";
        int imageWidth = 0, imageHeight = 0;
        if (i %= 100) {
            emit newActivities(activities);
            activities.clear();
        }

        Activity *activity = new Activity(path, type, lastModified, mimeType, QSize(imageWidth, imageHeight));
        activities.append(activity);
    }

    if (!activities.isEmpty()) {
        emit newActivities(activities);
    }
    emit searchFinished();
}

bool BalooSource::isBalooInitialized()
{
    Baloo::IndexerConfig config;
    return config.fileIndexingEnabled();
}

void BalooSource::balooConfigure()
{
    QStringList args;
    args << "kcm_baloofile";
    KToolInvocation::kdeinitExec("kcmshell4", args);
}
